﻿//
//  ICCanvasClip.as
//  InfiniteCanvas (Flash Viewer)
//
//  Created by Markus Mueller <markus@mindnode.com>.
//  http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import de.alex_uhlmann.animationpackage.*;
import de.alex_uhlmann.animationpackage.animation.*;
import de.alex_uhlmann.animationpackage.drawing.*;
import de.alex_uhlmann.animationpackage.utility.*;
import com.robertpenner.easing.*;


class ICCanvas extends MovieClip {// this class manages animations and mouse input
	var _aF:ICFrame = null; // activeFrame
	var _aT:ICNavigationTrail = null;	// activeTrail
	var _tF:ICFrame = null; // targetFrame
	var _cW:Number = 0; //canvasWidth
	var _cH:Number = 0; //canvasHeight
	var _history:Array = null;
	var _fC:MovieClip = null;
	var _running:Boolean = false;
	var _f:Boolean = true;
	var _hint:Boolean = true;
	var _hotSpots:Array = null;
	var _listener:Object = null;
	private var _controller:ICController = null;
	
	// called after the canvas mc has been created
	// inits some vars
	function initMovieClip(controller:ICController) {
		_controller = controller;
		_cW = _controller._canvasWidth;
		_cH = _controller._canvasHeight;
		_history = new Array();
		_hotSpots = new Array();
		
		// animation package init
		APCore.initialize();

		_listener = new Object();
		var owner:ICCanvas = this;
		
		_listener.onFinish = function(source, value) {
			_controller._fadeClip.clear();
			_controller._fadeClip._visible = false;

			this.stop();
			owner.finished();
		}
		
		_listener.fadeBackIn = function(source, value) {
			owner.center(owner._tF);
			this.stop();
			
			var alpha:Alpha;
			if (_aT.getEasing() == "CubicInOut") {
				alpha = new Alpha(owner._controller._fadeClip, [100, 0], owner._aT.getDuration() / 2, Cubic.easeInOut, "onFinish");
			} else {
				alpha = new Alpha(owner._controller._fadeClip, [100, 0], owner._aT.getDuration() / 2, "onFinish");
			}
			
			alpha.run();
		}
		APCore.addListener(_listener);
	}
	
	// called when every thing is ready for displaying the canvas
	// centers canvasContainer, finds and centers the first frame 
	public function startDisplaying(firstFrame:ICFrame) {
		_aF = firstFrame;
		_hint = _controller._document.getHotSpotHint();
		this.center(_aF);
		_controller.updateFrameNavigationButtons(((_history.length > 0) ? true : false), ((_aF.getTrailCount() > 0) ? true : false));
	}
	
	public function center(frame:ICFrame) {
		this._x = -frame.getX();
		this._y = -frame.getY();
		
		if (_controller._document.getScaleCanvas() == true) {
			_controller._canvasContainerScaling._xscale = _cW/frame.getWidth() * 100; // parent == canvasContainer!!!
			_controller._canvasContainerScaling._yscale = _cH/frame.getHeight() * 100; // parent == canvasContainer!!!
		} else {
			_controller._canvasContainerScaling._xscale = 100;
			_controller._canvasContainerScaling._yscale = 100;
		}
		_controller._canvasContainerRotation._rotation = frame.getRotation();
		
		// attach hot spots
		this.addHotSpots();
	}
	
	public function startAnimation(forward:Boolean, hotSpot:ICFrame) {
		if (_running == false) {
			if (forward) {				
				// check for active component hit
				var tmpFrame:ICFrame = null;
				if (hotSpot == null) {
					tmpFrame = _aF; // no hit
				} else {
					tmpFrame = hotSpot;
				}
				
				// check if there is a url
				tmpFrame.openURL();
				
				// get trail
				_aT = tmpFrame.getTrail();		
				if (_aT == null) return;
				_tF = _controller._document.getFrameWithID(_aT.getTargetFrameID());
				if (_tF == null) return;
				
				// save for back
				_history.push(_aF);
				_history.push(_aT);
			} else {
				// back in history
				if (_history.length < 2) return;
				_aT = ICNavigationTrail(_history.pop());
				_tF = ICFrame(_history.pop());
			}
	
			_running = true;
			_f = forward;
			_controller.updateFrameNavigationButtons(false, false);
			this.deleteHotSpots();

			switch (_aT.getType()) {
				case "movement": {
					this.moveTransition();
					break;
				}
				case "fade" : {
					this.fadeThroughColorTransition();
					break;
				}
				default : {
					// no animation
					this.noTransition();
				}
			}
		}
	}
	
	private function moveTransition() {
		var animation:Parallel = new Parallel();
		
		// movement
		animation.addChild(_aT.getTrail(_controller._canvas));
		
		// rotation
		var rotation:Rotation = null;
		if (_f) { // forward
			rotation = new Rotation(_controller._canvasContainerRotation,[_aF.getRotation(), _tF.getRotation()]);
		} else { // backward
			rotation = new Rotation(_controller._canvasContainerRotation,[_tF.getRotation(), _aF.getRotation()]);
		}
		animation.addChild(rotation);
		
		// scale
		if (_controller._document.getScaleCanvas() == true) {
			var scale:Scale = null;
			if (_f) { // forward
				scale = new Scale(_controller._canvasContainerScaling, _cW/_tF.getWidth() * 100, _cH/_tF.getHeight() * 100);
			} else { // backward
				scale = new Scale(_controller._canvasContainerScaling, [_cW/_tF.getWidth() * 100, _cH/_tF.getHeight() * 100, _cW/_aF.getWidth() * 100, _cH/_aF.getHeight() * 100]);
			}
			animation.addChild(scale);
		}
		
		if (_aT.getEasing() == "QuadInOut") {
			//animation.animationStyle(2000,Cubic.easeInOut,"onFinish");
			animation.animationStyle(_aT.getDuration(), Quad.easeInOut, "onFinish");
		} else {
			animation.animationStyle(_aT.getDuration(), "onFinish");
		}

		if (_f) { // forward
			animation.animate(0,100);
		} else { // backward
			animation.animate(100,0);
		}
		animation.forceEnd(false);
	}
	
	private function fadeThroughColorTransition() {
		var fade:MovieClip = _controller._fadeClip;
		fade.clear();
		fade.moveTo(0, 0);
		fade.beginFill(Number(_aT.getColor()));
		fade.lineTo(_cW, 0);
		fade.lineTo(_cW, _cH);
		fade.lineTo(0, _cH);
		fade.endFill();
				
		fade._alpha = 0;
		fade._visible = true;
				
		var alpha:Alpha;
		if (_aT.getEasing() == "CubicInOut") {
			alpha = new Alpha(fade, [0, 100], _aT.getDuration() / 2, Cubic.easeInOut, "fadeBackIn");
		} else {
			alpha = new Alpha(fade, [0, 100], _aT.getDuration() / 2, "fadeBackIn");
		}
		
		alpha.run();
	}
	
	private function noTransition() {
		this.finished();
	}
	
	public function finished() {
		_running = false;
		_aT = null;
		_aF = _tF;
		this.center(_aF);

		// check if we continue with animation
		if (_aF.getDelay() != -1) {
			// check if there is a url
			_aF.openURL();
			// start with next transition
			this.startAnimation(_f);
		} else {
			_controller.updateFrameNavigationButtons(((_history.length > 0) ? true : false), ((_aF.getTrailCount() > 0) ? true : false));
		}
	}
	
	private function deleteHotSpots() {
		while (_hotSpots.pop()) {
			_hotSpots.pop().removeMovieClip();
		}
	}
	
	private function addHotSpots() { //TODO  case no scale!!!
		this.deleteHotSpots();
		var hotSpots:Array = _aF.getHotSpots();
		for (var i:Number = 0; i < hotSpots.length; i++) {
			var hotSpotMC:MovieClip = _controller._canvasHotSpots.createEmptyMovieClip("hotspot"+i, _controller._canvasHotSpots.getNextHighestDepth());
			var hotSpot:ICFrame = hotSpots[i];
			var x, y, w, h:Number;
			
			if (_controller._document.getScaleCanvas() == true) { 
				// scaling
				w = hotSpot.getWidth() * (_cW / _aF.getWidth()); 
				h = hotSpot.getHeight() * (_cW / _aF.getHeight());
				x = hotSpot.getX() * (_cW / _aF.getWidth());
				y = _cH - hotSpot.getY() * (_cH / _aF.getHeight()) - h;
			} else {
				// no scaling
				w = hotSpot.getWidth();
				h = hotSpot.getHeight();
				x = (_cW / 2.0) - (_aF.getWidth() / 2.0) + hotSpot.getX();
				y = (_cH / 2.0) - (_aF.getHeight() / 2.0) + _aF.getHeight() - hotSpot.getY() - h;
			}

			with(hotSpotMC) {
				moveTo(x, y);
				beginFill(0x00ffff);
				lineTo(x+w, y);
				lineTo(x+w, y+h);
				lineTo(x, y+h);
				endFill();
			}
			hotSpotMC._alpha = 0;
			var owner:ICCanvas = this;
			hotSpotMC.useHandCursor = true;
			_hotSpots.push(hotSpotMC);
			_hotSpots.push(hotSpot);

			if (_hint == false) {
				hotSpotMC.useHandCursor = false;
			} else {
				hotSpotMC.useHandCursor = true;
			}

			hotSpotMC.onPress = function() {
				var i:Number = Number(this._name.slice(7)) * 2 + 1;
				owner.startAnimation(true, owner._hotSpots[i]);
			};
		}
	}
}
