﻿//
//  ICController
//  InfiniteCanvas (Viewer)
//
//  Created by Markus Mueller <markus@mindnode.com>.
//  http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

class ICController {
	public var _canvas:ICCanvas = null;
	public var _canvasBG:MovieClip = null;
	public var _canvasContainerScaling:MovieClip = null;
	public var _canvasContainerRotation:MovieClip = null;
	public var _canvasHotSpots = null;
	public var _canvasWidth:Number = null;
	public var _canvasHeight:Number = null;
	
	private var _stage:MovieClip = null;
	private var _useButtons:Boolean = true;
	private var _path:String = null;
	private var _pB:MovieClip = null;
	
	public var _loader:MovieClipLoader = null;
	public var _loadCount:Number = -1;
	public var _loadStatus:Number = 0;
	public var _document:ICDocument = null;
	private var _parser:ICParser = null;

	private var _buttonLabel:MovieClip = null;
	
	public var _fadeClip:MovieClip = null;

	
	function ICController(stage:MovieClip, path:String, useButtons:Boolean) {
		// prepare the stage. this is VERY important if authors use sizes != document size
		Stage.scaleMode = "noScale";
		Stage.align = "TL";
		
		_stage = stage;
		_path = path;
		_canvasWidth = Stage.width;
	 	_canvasHeight = Stage.height

		_useButtons = useButtons;
		if (useButtons == true) {
			_canvasHeight -= 40;
		}

		createCanvas();
		createProgressBar();
		
		startLoading();
	}
	
	function createCanvas() {
		// canvas background
		_canvasBG = _stage.createEmptyMovieClip("canvasBG", _stage.getNextHighestDepth());
		
		// canvas container - needed for scaling and rotating
		_canvasContainerScaling = _stage.createEmptyMovieClip("canvasContainerScaling", _stage.getNextHighestDepth());
		_canvasContainerRotation = _canvasContainerScaling.createEmptyMovieClip("canvasContainerRotation", _stage.getNextHighestDepth());

		// the canvas
		_canvasContainerRotation.attachMovie("ICCanvas", "canvas", 1);
		_canvas = _canvasContainerRotation.canvas;
		_canvas.initMovieClip(this);
		
		// canvas mask
		var canvasMask:MovieClip = _stage.createEmptyMovieClip("canvasMask", _stage.getNextHighestDepth());
		canvasMask.moveTo(0, 0);
		canvasMask.beginFill(0x000000);
		canvasMask.lineTo(_canvasWidth, 0);
		canvasMask.lineTo(_canvasWidth, _canvasHeight);
		canvasMask.lineTo(0, _canvasHeight);
		canvasMask.endFill();
			
		_canvasContainerScaling.setMask(canvasMask);
		_canvasHotSpots = _stage.createEmptyMovieClip("canvasHotSpots", _stage.getNextHighestDepth());
		
		// needed for fade transition
		_fadeClip = _stage.createEmptyMovieClip("fadeClip", _stage.getNextHighestDepth());
		_fadeClip._visible = false;
		
		// hide canvas during loading
		_canvasBG._visible = false;
		_canvasContainerScaling._visible = false;
	}
	
	
	private function createProgressBar() {
		// calculate size / location
		var pbX, pbY, pbWidth, pbHeight:Number;
		if (_canvasWidth>300) {
			pbX = _canvasWidth/2-125;
			pbWidth = 250;
		} else {
			pbX = 10;
			pbWidth = _canvasWidth-20;
		}
		pbY = _canvasHeight/2-3;
		pbHeight = 6;
	
		// create progress bar
		_pB = _stage.createEmptyMovieClip("progressBar", _stage.getNextHighestDepth());
		_pB.createEmptyMovieClip("bar", _pB.getNextHighestDepth());
		_pB.createEmptyMovieClip("vBar", _pB.getNextHighestDepth());
		_pB.createEmptyMovieClip("stroke", _pB.getNextHighestDepth());
		_pB.createTextField("txt", _pB.getNextHighestDepth(), 0, pbHeight, pbWidth, 22);
		_pB._x = pbX;
		_pB._y = pbY;
		
		with (_pB.bar) {
			beginFill(0x00FF00);
			moveTo(0, 0);
			lineTo(pbWidth, 0);
			lineTo(pbWidth, pbHeight);
			lineTo(0, pbHeight);
			lineTo(0, 0);
			endFill();
			_xscale = 0;
		}
		
		with (_pB.vBar) {
			lineStyle(1, 0x000033);
			moveTo(0, 0);
			lineTo(0, pbHeight);
		}
		
		with (_pB.stroke) {
			lineStyle(1, 0x000000);
			moveTo(0, 0);
			lineTo(pbWidth, 0);
			lineTo(pbWidth, pbHeight);
			lineTo(0, pbHeight);
			lineTo(0, 0);
		}
	}
	
	private function startLoading() {
		_loader = new MovieClipLoader();
		var mclListener:Object = new Object();
		var owner:ICController = this;
		mclListener.onLoadInit = function(targetMC:MovieClip) {
			// mc fully loaded
			var c:ICMovieClip = ICMovieClip(targetMC._parent);
			c._visible = true;
			c.initPanel();
			
			// update the progressbar
			owner._loadStatus++;
			owner.updateProgressBar();
			owner.checkIfReady();
		};
		_loader.addListener(mclListener);

		// start loading
		_parser = new ICParser(_path, this);
	}
	
	public function updateProgressBar():Void {
		var pos:Number = Math.round(_loadStatus / _loadCount * 100);
		_pB.bar._xscale = pos;
		_pB.vBar._x = _pB.bar._width;
		_pB.txt.text = pos+"%";
	}

	public function printLoadingErrorMessage(msg) {
		_pB.txt.text = "Error: " + msg;
		trace(msg);
	}

	private function safariTrace(str){
		//this.loadMovie(str);
	}

	public function checkIfReady() {
		if (_loadCount == _loadStatus) {
			_pB.removeMovieClip();
			_pB = null;
			_document = _parser.getDocument();
			_parser = null;

			if (_document.getNavigationMode() == 0) {
				//gotoAndPlay(3);
			} else if (_document.getNavigationMode() == 1) {
				initFrameNavigation();
				startDisplaying();
			}
		}
	}
	
	public function drawArrow(mc:MovieClip, flip:Boolean, selected:Boolean) {
		with (mc) {
			moveTo(-5, -5);
			beginFill(0xFFFFFF);
			lineTo(25, -5);
			lineTo(25, 20);
			lineTo(-5, 20);
			endFill();
	
			if (selected)
				lineStyle(3, 0x000000);
			else
				lineStyle(3, 0x999999);
	
			if (flip) {
				moveTo(5, 0);
				lineTo(0, 5);
				lineTo(5, 10);
				moveTo(15, 5);
				lineTo(0, 5);
			} else {
				moveTo(10, 0);
				lineTo(15, 5);
				lineTo(10, 10);
				moveTo(15, 5);
				lineTo(0, 5);
			}
		}
	}

	private function initFrameNavigation() {
		if (_useButtons == true) {
			// create buttons
			_buttonLabel = _stage.createEmptyMovieClip("label", _stage.getNextHighestDepth());
			var line:MovieClip = _buttonLabel.createEmptyMovieClip("line", 1);
	
			var back:MovieClip = _buttonLabel.createEmptyMovieClip("back", 2);
			back.createEmptyMovieClip("sOn", 0);// state On
			back.createEmptyMovieClip("sOff", 1);
	
			var forward:MovieClip = _buttonLabel.createEmptyMovieClip("forward", 3);
			forward.createEmptyMovieClip("sOn", 0);
			forward.createEmptyMovieClip("sOff", 1);
	
			_buttonLabel._x = 0;
			_buttonLabel._y = _canvasHeight;
	
			line.lineStyle(1, 0x000000);
			line.moveTo(0, 1);
			line.lineTo(_canvasWidth, 1);
	
			drawArrow(_buttonLabel.back.sOn, true, true);
			_buttonLabel.back.sOn._visible = false;
			drawArrow(_buttonLabel.back.sOff, true, false);
		
			_buttonLabel.back._x = 10;
			_buttonLabel.back._y = 15;

			var owner:ICController = this;
			_buttonLabel.back.sOn.onPress = function() {
				owner._canvas.startAnimation(false);
			};
			
	
			drawArrow(_buttonLabel.forward.sOn, false, true);
			_buttonLabel.forward.sOff._visible = false;
			drawArrow(_buttonLabel.forward.sOff, false, false);
	
			_buttonLabel.forward._x = _canvasWidth-15-15;
			_buttonLabel.forward._y = 15;

			_buttonLabel.forward.sOn.onPress = function() {
				owner._canvas.startAnimation(true);
			};
		}

		// mouse listener
		// canvasBG is also used for mouse down 
		//(we use it because it is on a lower level than canvas. 
		//we can't use canvas, because it only recieves events when there is a panel or a trail
		_canvasBG.useHandCursor = false;
		var owner:ICController = this;
		_canvasBG.onPress = function() {
			owner._canvas.startAnimation(true);
		};
	}

	public function updateFrameNavigationButtons(back:Boolean, forward:Boolean) {
		_buttonLabel.back.sOn._visible = back;
		_buttonLabel.back.sOff._visible = !back;
	
		_buttonLabel.forward.sOn._visible = forward;
		_buttonLabel.forward.sOff._visible = !forward;
	}
	
	private function startDisplaying() {
		// center startframe and start displaying the comic		
		// center around origin
		_canvasContainerScaling._x = _canvasWidth / 2.0;
		_canvasContainerScaling._y = _canvasHeight / 2.0;
		_canvasBG._visible = true;
		_canvasContainerScaling._visible = true;
		_canvas.startDisplaying(_document.getFirstFrame());
	}

}
