﻿//
//  ICNavigationTrail.as
//  InfiniteCanvas (Flash Viewer)
//
//  Created by Markus Mueller <markus@mindnode.com>.
//  http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

import de.alex_uhlmann.animationpackage.*;
import de.alex_uhlmann.animationpackage.animation.*;
import de.alex_uhlmann.animationpackage.drawing.*;
import de.alex_uhlmann.animationpackage.utility.*;
import com.robertpenner.easing.*;

class ICNavigationTrail {
	// Represents a Navigation Trail.
	var _type:String = "none";
	var _targetFrameID:Number = 0;
	var _duration:Number = 3;
	var _components:Array = null;
	var _easing:String = null;
	var _color:String = "0x000000";
	
	function ICNavigationTrail(type:String, targetFrame:Number, duration:Number) {
		_type = type;
		_targetFrameID = targetFrame;
		_duration = duration;
		_components = new Array();
		_easing = "QuadInOut";
	}
	
	
	public function getType():String {
		return _type;
	}
	
	public function getTargetFrameID():Number {
		return _targetFrameID;
	}
	
	public function getDuration() {
		return _duration;
	}
	
	public function setEasing(easing) {
		_easing = easing;
	}
	
	public function getEasing():String {
		return _easing;
	}
	
	public function setColor(color:String) {
		_color = color;
	}
	
	public function getColor():String {
		return _color;
	}
	
	public function addLine(x:Number, y:Number) {
		var newComponent:Object = new Object();
		newComponent.type = "line";
		newComponent.x = x;
		newComponent.y = y;
		
		_components.push(newComponent);
	}
	
	public function addCurve(c1X:Number, c1Y:Number, c2X:Number, c2Y:Number, x:Number, y:Number) {
		var newComponent:Object = new Object();
		newComponent.type = "curve";
		newComponent.x = x;
		newComponent.y = y;
		newComponent.c1X = c1X;
		newComponent.c1Y = c1Y;
		newComponent.c2X = c2X;
		newComponent.c2Y = c2Y;
		
		_components.push(newComponent);
	}
	
	public function getTrail(canvas:ICCanvas) {
		var object:Object = null;
		var trail:MoveOnPath = new MoveOnPath(canvas);
		
		if (canvas._f) {
			var x:Number = canvas._aF.getX();
			var y:Number = canvas._aF.getY();
		} else {
			var x:Number = canvas._tF.getX();
			var y:Number = canvas._tF.getY();
		}
		
		for (var i:Number = 0; i < _components.length; i++) {
			object = _components[i];
			if  (object.type == "line") {
				trail.addChild(new MoveOnQuadCurve(canvas, -x, -y, -x+0.1, -y+0.1, -object.x, -object.y));
			} else {
				trail.addChild(new MoveOnQuadCurve(canvas, -x, -y, -object.c1X, -object.c1Y, -object.x, -object.y));
			}

			x = object.x;
			y = object.y;
		}

		trail.orientToPath(false);
		trail.forceEnd(false);
		
		return trail;
	}
	
	public function addTrailComponent(newComponent:ICTrailComponent) {
		_components.push(newComponent);
	}
}